package com.vendor.burn.packagers.nand.ip;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.define.IFastbootProgressMonitor;
import com.vendor.burn.model.devices.unknown.UnknownDevice;
import com.vendor.burn.packagers.nand.FlashOption;
import com.vendor.burn.packagers.nand.NandCRandom;
import com.vendor.burn.packagers.nand.PageType;
import com.vendor.burn.packagers.nand.ecc.ECC610;
import com.vendor.burn.packagers.nand.ecc.ECC620;
import com.vendor.burn.packagers.nand.ecc.ECCBase;
import com.vendor.burn.packagers.nand.ecc.ECCType;
import com.vendor.burn.packagers.nand.ip.INandSetting;
import com.vendor.burn.packagers.nand.ip.NandIP500;
import com.vendor.burn.packagers.nand.ip.NandIP600;
import com.vendor.burn.packagers.nand.ip.NandIP610;
import com.vendor.burn.packagers.nand.ip.NandIP620;
import com.vendor.burn.utilitily.BytesUtils;
import com.vendor.burn.utilitily.FileUtils;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.List;
import org.apache.commons.io.FileUtils;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

public class NandIPCommon implements INandSetting {
  public static final String NAND = "Nand";
  
  public static final String SPINAND = "SPINand";
  
  public static final int NandIP_500 = 500;
  
  public static final int NANDIP_600 = 600;
  
  public static final int NANDIP_610 = 610;
  
  public static final int NandIP_611 = 611;
  
  public static final int NANDIP_620 = 620;
  
  protected static final int REVERSE_DATA_LENTH = 32;
  
  private static final String[] PIN_SAVE_MODE_ARRAY = new String[] { "v2", "v3" };
  
  protected static final int DEFAULT_OOB_SIZE = 32;
  
  protected String curSelectedNandWay;
  
  protected byte[] genPageDataBytes(ECCBase ecc, FlashOption option, byte[] data, int ctrlSize, int eccSize) {
    byte[] result = BytesUtils.genBytesWithFF(option.getPageType()
        .getPageSize() + option.getOOBSize());
    int pageSize = option.getPageType().getPageSize();
    int totalLen = pageSize + 2 + ctrlSize;
    int partCount = pageSize / 1024;
    int partSize = totalLen / partCount;
    int i;
    for (i = 0; i < partCount; i++) {
      byte[] part = new byte[partSize];
      byte[] partForEcc = new byte[partSize];
      BytesUtils.copyBytes(part, data, 0, i * partSize, partSize);
      BytesUtils.copyBytes(partForEcc, data, 0, i * partSize, partSize);
      byte[] eccBuffer = BytesUtils.genBytesWithFF(eccSize);
      ECCType type = option.getEccType();
      if (type == ECCType.Ecc4bit) {
        ecc.ecc_4bit_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc16bit1k) {
        ecc.ecc_16bit1k_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc24bit1k) {
        ecc.ecc_24bit1k_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc40bit1k) {
        ecc.ecc_40bit1k_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc64bit1k) {
        ecc.ecc_64bit1k_gen(partForEcc, partSize, eccBuffer);
      } else {
        return null;
      } 
      if (i < partCount - 1) {
        BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, partSize);
        BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + partSize, 0, eccSize);
      } else {
        int lastPartDataSize = partSize - 2 - ctrlSize;
        BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, lastPartDataSize);
        BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + lastPartDataSize, 0, eccSize);
        BytesUtils.copyBytes(result, part, i * (partSize + eccSize) + lastPartDataSize + eccSize, lastPartDataSize + 2, ctrlSize);
      } 
    } 
    for (i = result.length - 1; i > pageSize + 1; i--)
      result[i] = result[i - 2]; 
    result[pageSize] = data[pageSize];
    result[pageSize + 1] = data[pageSize + 1];
    return result;
  }
  
  protected byte[] genPageDataBytesFor610(ECC610 ecc, FlashOption option, byte[] data, int ctrlSize, int eccSize) {
    byte[] result = BytesUtils.genBytesWithFF(option.getPageType()
        .getPageSize() + option.getOOBSize());
    int pageSize = option.getPageType().getPageSize();
    int totalLen = pageSize + 2 + ctrlSize;
    int partCount = pageSize / 1024;
    int partSize = totalLen / partCount;
    int i;
    for (i = 0; i < partCount; i++) {
      byte[] part = new byte[partSize];
      byte[] partForEcc = new byte[partSize];
      BytesUtils.copyBytes(part, data, 0, i * partSize, partSize);
      BytesUtils.copyBytes(partForEcc, data, 0, i * partSize, partSize);
      byte[] eccBuffer = BytesUtils.genBytesWithFF(eccSize);
      ECCType type = option.getEccType();
      if (type == ECCType.Ecc4bit) {
        ecc.ecc_8bit_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc13bit) {
        ecc.ecc_13bit_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc24bit) {
        ecc.ecc_24bit1k_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc27bit) {
        ecc.ecc_27bit1k_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc41bit) {
        ecc.ecc_41bit1k_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc60bit) {
        ecc.ecc_60bit1k_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc80bit) {
        ecc.ecc_80bit1k_gen(partForEcc, partSize, eccBuffer);
      } else {
        return null;
      } 
      if (i < partCount - 1) {
        BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, partSize);
        BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + partSize, 0, eccSize);
      } else {
        int lastPartDataSize = partSize - 2 - ctrlSize;
        BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, lastPartDataSize);
        BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + lastPartDataSize, 0, eccSize);
        BytesUtils.copyBytes(result, part, i * (partSize + eccSize) + lastPartDataSize + eccSize, lastPartDataSize + 2, ctrlSize);
      } 
    } 
    for (i = result.length - 1; i > pageSize + 1; i--)
      result[i] = result[i - 2]; 
    result[pageSize] = data[pageSize];
    result[pageSize + 1] = data[pageSize + 1];
    return result;
  }
  
  protected byte[] genPageDataBytesFor620(ECC620 ecc, FlashOption option, byte[] data, int ctrlSize, int eccSize) {
    byte[] result = BytesUtils.genBytesWithFF(option.getPageType()
        .getPageSize() + option.getOOBSize());
    int pageSize = option.getPageType().getPageSize();
    int totalLen = pageSize + 2 + ctrlSize;
    int partCount = pageSize / 1024;
    int partSize = totalLen / partCount;
    for (int i = 0; i < partCount; i++) {
      byte[] part = new byte[partSize];
      byte[] partForEcc = new byte[partSize];
      BytesUtils.copyBytes(part, data, 0, i * partSize, partSize);
      BytesUtils.copyBytes(partForEcc, data, 0, i * partSize, partSize);
      byte[] eccBuffer = BytesUtils.genBytesWithFF(eccSize);
      ECCType type = option.getEccType();
      if (type == ECCType.Ecc4bit) {
        ecc.ecc_4bit_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc8bit512) {
        ecc.ecc_8bit_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc24bit1k) {
        ecc.ecc_24bit1k_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc40bit1k) {
        ecc.ecc_40bit1k_gen(partForEcc, partSize, eccBuffer);
      } else if (type == ECCType.Ecc64bit1k) {
        ecc.ecc_64bit1k_gen(partForEcc, partSize, eccBuffer);
      } else {
        return null;
      } 
      if (option.getPageType() == PageType.Size16K) {
        if (option.getEccType() == ECCType.Ecc40bit1k)
          if (i < partCount - 2) {
            BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, partSize);
            BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + partSize, 0, eccSize);
          } else if (i == partCount - 2) {
            BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, partSize);
            BytesUtils.copyBytes(result, data, i * (partSize + eccSize) + partSize + 14, pageSize, 2);
            BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + partSize, 0, 14);
            BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + partSize + 2 + 14, 14, 56);
          } else {
            int lastPartDataSize = partSize - 2 - ctrlSize;
            BytesUtils.copyBytes(result, part, i * (partSize + eccSize) + 2, 0, lastPartDataSize);
            BytesUtils.copyBytes(result, data, i * (partSize + eccSize) + 2 + lastPartDataSize + eccSize, pageSize + 2, ctrlSize);
            BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + 2 + lastPartDataSize, 0, eccSize);
          }  
        if (option.getEccType() == ECCType.Ecc64bit1k)
          if (i < partCount - 2) {
            BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, partSize);
            BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + partSize, 0, eccSize);
          } else if (i == partCount - 2) {
            BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, 452);
            BytesUtils.copyBytes(result, data, i * (partSize + eccSize) + 452, pageSize, 2);
            BytesUtils.copyBytes(result, part, i * (partSize + eccSize) + 452 + 2, 452, 574);
            BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + 452 + 2 + 574, 0, 112);
          } else {
            int lastPartDataSize = partSize - 2 - ctrlSize;
            BytesUtils.copyBytes(result, part, i * (partSize + eccSize) + 2, 0, lastPartDataSize);
            BytesUtils.copyBytes(result, data, i * (partSize + eccSize) + 2 + lastPartDataSize + eccSize, pageSize + 2, ctrlSize);
            BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + 2 + lastPartDataSize, 0, eccSize);
          }  
      } else if (option.getPageType() == PageType.Size8K && option
        .getEccType() == ECCType.Ecc64bit1k) {
        if (i < partCount - 1) {
          BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, partSize);
          BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + partSize, 0, eccSize);
        } else {
          BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, 212);
          BytesUtils.copyBytes(result, data, i * (partSize + eccSize) + 212, pageSize, 2);
          BytesUtils.copyBytes(result, part, i * (partSize + eccSize) + 212 + 2, 212, 784);
          BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + 212 + 2 + 784, 0, eccSize);
          BytesUtils.copyBytes(result, data, i * (partSize + eccSize) + 212 + 2 + 784 + 112, pageSize + 2, ctrlSize);
        } 
      } else if (i < partCount - 1) {
        BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, partSize);
        BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + partSize, 0, eccSize);
      } else {
        int lastPartDataSize = partSize - 2 - ctrlSize;
        BytesUtils.copyBytes(result, part, i * (partSize + eccSize), 0, lastPartDataSize);
        BytesUtils.copyBytes(result, eccBuffer, i * (partSize + eccSize) + lastPartDataSize, 0, eccSize);
        BytesUtils.copyBytes(result, part, i * (partSize + eccSize) + lastPartDataSize + eccSize, lastPartDataSize + 2, ctrlSize);
      } 
    } 
    boolean isSomeOption = (option.getPageType() != PageType.Size16K || (option.getEccType() != ECCType.Ecc40bit1k && option.getEccType() != ECCType.Ecc64bit1k));
    if (isSomeOption) {
      boolean needInsertBB = (option.getPageType() != PageType.Size8K || option.getEccType() != ECCType.Ecc64bit1k);
      if (needInsertBB) {
        for (int j = result.length - 1; j > pageSize + 1; j--)
          result[j] = result[j - 2]; 
        result[pageSize] = data[pageSize];
        result[pageSize + 1] = data[pageSize + 1];
      } 
    } 
    return result;
  }
  
  public void make(String outputPath, FlashOption option, List<PartitionItem> partitions, int blockSize, boolean isSelectRandom, IFastbootProgressMonitor fastbootMonitor, IProgressMonitor monitor) {
    fastbootMonitor.infoln("\nStart to make nand burner image...");
    fastbootMonitor.infoln("Options: " + option.toString() + "\n");
    monitor.beginTask(Messages.NandImageDialog_Title, partitions.size());
    int randomPageNum = -1;
    if (blockSize > -1)
      randomPageNum = blockSize / 8192; 
    label86: for (PartitionItem item : partitions) {
      if (monitor.isCanceled())
        return; 
      boolean itemState = (!item.isChecked() || item.getFile() == null);
      if (itemState) {
        monitor.worked(1);
        continue;
      } 
      differenceBlockSize(item, blockSize);
      fastbootMonitor.info("Make image for '" + item.getFile().getName() + "'...");
      int pageCheckSize = option.getPageType().getPageSize();
      if (item.getFileSystem() == FileSystem.yaffs)
        pageCheckSize = option.getPageType().getPageSize() + option.getOOBSize(); 
      File file = (item.getNewTempFile() == null) ? item.getFile() : item.getNewTempFile();
      int pageCount = (int)((file.length() + pageCheckSize - 1L) / pageCheckSize);
      FileSystem fileSystem = item.getFileSystem();
      if (fileSystem == FileSystem.yaffs)
        pageCount = (int)(file.length() / pageCheckSize); 
      File outFile = FileUtils.getFile(new String[] { outputPath + outputPath + File.separator + ".burn" });
      if (outFile.exists())
        FileUtils.deleteFile(outFile); 
      RandomAccessFile inRanFile = null;
      RandomAccessFile outRanFile = null;
      boolean ret = true;
      try {
        FileUtils.checkFilePath(file);
        inRanFile = new RandomAccessFile(file, "r");
        outRanFile = new RandomAccessFile(outFile, "rw");
        int pageMakeSize = option.getPageType().getPageSize() + option.getOOBSize();
        for (int i = 0; i < pageCount; i++) {
          if (monitor.isCanceled()) {
            closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
            FileUtils.deleteFile(item.getNewTempFile());
            if (!ret) {
              FileUtils.deleteFile(outFile);
              monitor.worked(1);
              continue label86;
            } 
            return;
          } 
          byte[] pagebuf = BytesUtils.genBytesWithFF(pageMakeSize);
          try {
            inRanFile.seek((i * pageCheckSize));
            int read = inRanFile.read(pagebuf, 0, pageCheckSize);
            if (read < 0)
              System.out.println("Read Error."); 
            if (randomPageNum == -1)
              makeEmtpyFlag(option, pagebuf); 
          } catch (IOException e) {
            fastbootMonitor.warnln("FAILED! (Read data error.)");
            ret = false;
            break;
          } 
          byte[] data = null;
          try {
            data = genPageData(pagebuf, option, option.getOOBSize());
            if (randomPageNum > -1) {
              NandCRandom random = new NandCRandom();
              random.page_random_gen(data, option.getPageType().getPageSize(), option.getOOBSize(), i % randomPageNum);
            } 
            outRanFile.seek((i * pageMakeSize));
            outRanFile.write(data);
          } catch (Exception e) {
            fastbootMonitor.warnln("FAILED! (Data convert error.)");
            ret = false;
            break;
          } 
        } 
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } catch (FileNotFoundException e) {
        fastbootMonitor.warnln("FAILED! (Source file not found.)");
        monitor.worked(1);
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
        } 
        continue;
      } catch (IOException iOException) {
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } finally {
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } 
      fastbootMonitor.info("OK!");
      fastbootMonitor.warnln("(Start Address = 0x" + 
          Long.toHexString(item.getStart()) + ", Length = 0x" + 
          Long.toHexString(item.getLength()) + ")");
      monitor.worked(1);
    } 
    fastbootMonitor
      .infoln("\nImage creating operation ends. All of the selected partitions are processed.");
    monitor.done();
  }
  
  public void makeEmtpyFlag(FlashOption option, byte[] pagebuf) {
    int emptyFlagIndex;
    if (option.getPageType() == PageType.Size2K && option
      .getEccType() == ECCType.Ecc1bit) {
      emptyFlagIndex = option.getPageType().getPageSize() + NandIP500.getOobfree2k1bit()[0] + NandIP500.getNfc504EpmPos();
    } else {
      emptyFlagIndex = option.getPageType().getPageSize() + NandIP500.getOobfreeDef()[0] + NandIP500.getNfc504EpmPos();
    } 
    pagebuf[emptyFlagIndex] = 0;
    pagebuf[emptyFlagIndex + 1] = 0;
  }
  
  protected void differenceBlockSize(PartitionItem item, int blockSize) {
    long length = item.getFile().length();
    int differenceValue = (int)length % blockSize;
    if (differenceValue != 0) {
      File oldFile = item.getFile();
      RandomAccessFile tempFile = null;
      try {
        String newFilePath = oldFile.getCanonicalPath() + ".temp";
        File newFile = new File(newFilePath);
        item.setNewTempFile(newFile);
        FileUtils.copy(null, oldFile, newFile);
        byte[] temp = BytesUtils.genBytesWithFF(blockSize - differenceValue);
        FileUtils.checkFilePath(newFile);
        tempFile = new RandomAccessFile(newFile, "rw");
        tempFile.seek(length);
        tempFile.write(temp);
      } catch (FileNotFoundException fileNotFoundException) {
      
      } catch (IOException iOException) {
      
      } finally {
        if (tempFile != null)
          try {
            tempFile.close();
          } catch (IOException iOException) {} 
      } 
    } 
  }
  
  protected void closeTransportStream(IFastbootProgressMonitor fastbootMonitor, RandomAccessFile inRanFile, RandomAccessFile outRanFile) {
    try {
      if (inRanFile != null)
        inRanFile.close(); 
    } catch (IOException e) {
      fastbootMonitor.warnln("FAILED! (Close file error.)");
    } 
    try {
      if (outRanFile != null)
        outRanFile.close(); 
    } catch (IOException e) {
      fastbootMonitor.warnln("FAILED! (Close file error.)");
    } 
  }
  
  public static com.vendor.burn.packagers.nand.ip.NandIPCommon chooseNandVersion(UnknownDevice device) {
    NandIP620 nandIP620;
    com.vendor.burn.packagers.nand.ip.NandIPCommon nandSet = null;
    if (device.getNandIPVer() == 500) {
      NandIP500 nandIP500 = new NandIP500();
    } else if (device.getNandIPVer() == 600) {
      NandIP600 nandIP600 = new NandIP600();
    } else if (device.getNandIPVer() == 610) {
      NandIP610 nandIP610 = new NandIP610();
    } else if (device.getNandIPVer() == 620) {
      nandIP620 = new NandIP620();
    } else {
      MessageDialog.openInformation(
          Display.getCurrent().getActiveShell(), "BurnTool", Messages.NandImage_DoNotSupport);
      return null;
    } 
    return (com.vendor.burn.packagers.nand.ip.NandIPCommon)nandIP620;
  }
  
  public static com.vendor.burn.packagers.nand.ip.NandIPCommon chooseSPINandVersion(UnknownDevice device) {
    NandIP620 nandIP620;
    com.vendor.burn.packagers.nand.ip.NandIPCommon nandSet = null;
    if (device.getSpinandIPVer() == 500) {
      NandIP500 nandIP500 = new NandIP500();
    } else if (device.getSpinandIPVer() == 600) {
      NandIP600 nandIP600 = new NandIP600();
    } else if (device.getSpinandIPVer() == 610) {
      NandIP610 nandIP610 = new NandIP610();
    } else if (device.getSpinandIPVer() == 620) {
      nandIP620 = new NandIP620();
    } else {
      MessageDialog.openInformation(
          Display.getCurrent().getActiveShell(), "BurnTool", Messages.NandImage_DoNotSupport);
      return null;
    } 
    return (com.vendor.burn.packagers.nand.ip.NandIPCommon)nandIP620;
  }
  
  public FlashOption[] getAvailableOptions(IBurner burner) {
    return null;
  }
  
  public byte[] genPageData(byte[] pagebuf, FlashOption option, int oobSize) {
    return null;
  }
  
  public String getCurSelectedNandWay() {
    return this.curSelectedNandWay;
  }
  
  public void setCurSelectedNandWay(String curSelectedNandWay) {
    this.curSelectedNandWay = curSelectedNandWay;
  }
  
  public static String[] getPinSaveModeArray() {
    return (String[])PIN_SAVE_MODE_ARRAY.clone();
  }
}
